@@ -1,4 +1,3 @@
-#  -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
 # Copyright 2020-2021 John Mille <john@ews-network.net>
 
@@ -8,9 +7,12 @@
 import json
 import re
 from copy import deepcopy
+from datetime import datetime as dt
+from uuid import uuid4
 
+import boto3
 import yaml
-from compose_x_common.compose_x_common import keyisset, keypresent
+from compose_x_common.compose_x_common import keyisset, keypresent, set_else_none
 
 try:
     from yaml import CLoader as Loader
@@ -21,6 +23,9 @@ from troposphere import AWS_NO_VALUE, GetAtt, Ref, Sub, Template
 
 from cfn_kafka_admin.models.admin import *
 
+DATE = dt.utcnow().isoformat()
+FILE_PREFIX = f'{dt.utcnow().strftime("%Y/%m/%d/%H%M")}/{str(uuid4().hex)[:6]}/'
+
 from .cfn_resources_definitions import KafkaAclPolicy
 from .cfn_resources_definitions.custom import KafkaAcl as CACLs
 from .cfn_resources_definitions.custom import KafkaTopic as CTopic
@@ -34,7 +39,7 @@ NONALPHANUM = re.compile(r"([^a-zA-Z0-9]+)")
 
 def merge_topics(final, override, extend_config_only=False):
     """
-    Function to override and update settings from override to primary
+    Function to override and update new_settings from override to primary
     Topics are filtered out via the Name property
     :param dict final:
     :param dict override:
@@ -43,7 +48,9 @@ def merge_topics(final, override, extend_config_only=False):
     :rtype: dict
     """
     if keyisset("Topics", override):
-        override_top_topics = Topics.parse_obj(override["Topics"]).dict()
+        override_top_topics = Topics.parse_obj(override["Topics"]).dict(
+            by_alias=True,
+        )
         if extend_config_only:
             # Allows to add the config and ensure that we do not import topics from config
             if keypresent("Topics", override_top_topics):
@@ -69,7 +76,7 @@ def merge_topics(final, override, extend_config_only=False):
 
 def merge_acls(final, override, extend_all=False):
     """
-    Function to override and update settings from override to primary
+    Function to override and update new_settings from override to primary
     All ACL policy is a dictionary made of simple objects, no key filtering
 
     :param dict final:
@@ -79,7 +86,7 @@ def merge_acls(final, override, extend_all=False):
     :rtype: dict
     """
     if keyisset("ACLs", override):
-        override_acls = ACLs.parse_obj(override["ACLs"]).dict()
+        override_acls = ACLs.parse_obj(override["ACLs"]).dict(by_alias=True)
         if keypresent("Policies", override_acls) and not extend_all:
             del override_acls["Policies"]
             final["ACLs"].update(override_acls)
@@ -88,14 +95,14 @@ def merge_acls(final, override, extend_all=False):
                 merged_lists = override_acls["Policies"] + final["ACLs"]["Policies"]
             else:
                 merged_lists = override_acls["Policies"]
-            acls = [dict(y) for y in set(tuple(x.items()) for x in merged_lists)]
+            acls = [dict(y) for y in {tuple(x.items()) for x in merged_lists}]
             final["ACLs"].update(override_acls)
             final["ACLs"]["Policies"] = acls
 
 
 def merge_contents(primary, override, extend_all=False):
     """
-    Function to override and update settings from override to primary
+    Function to override and update new_settings from override to primary
     :param primary:
     :param override:
     :param extend_all: Whether the policies or ACLs can be merged.
@@ -113,7 +120,7 @@ def merge_contents(primary, override, extend_all=False):
         and isinstance(override["Globals"], dict)
     ):
         override_globals = EwsKafkaParameters.parse_obj(override["Globals"])
-        final["Globals"].update(override_globals.dict())
+        final["Globals"].update(override_globals.dict(by_alias=True))
 
     if (
         keypresent("Schemas", final)
@@ -121,21 +128,21 @@ def merge_contents(primary, override, extend_all=False):
         and isinstance(override["Schemas"], dict)
     ):
         override_globals = Schemas.parse_obj(override["Schemas"])
-        final["Schemas"].update(override_globals.dict())
+        final["Schemas"].update(override_globals.dict(by_alias=True))
     elif (
         not keypresent("Schemas", final)
         and keyisset("Schemas", override)
         and isinstance(override["Schemas"], dict)
     ):
         override_globals = Schemas.parse_obj(override["Schemas"])
-        final["Schemas"] = override_globals.dict()
+        final["Schemas"] = override_globals.dict(by_alias=True)
 
     merge_acls(final, override, extend_all)
     merge_topics(final, override, not extend_all)
     return final
 
 
-class KafkaStack(object):
+class KafkaStack:
     """
     Class to represent the Kafka topics / acls / schemas in CloudFormation.
     """
@@ -152,26 +159,41 @@ class KafkaStack(object):
         final_content = {"Globals": {}, "Topics": {}, "ACLs": {}}
         for file_path in files_paths:
             if file_path.endswith(".yaml") or file_path.endswith(".yml"):
-                with open(file_path, "r") as file_fd:
+                with open(file_path) as file_fd:
                     file_content = file_fd.read()
                 yaml_content = yaml.load(file_content, Loader=Loader)
                 final_content = merge_contents(
                     final_content, yaml_content, extend_all=True
                 )
         if config_file_path:
-            with open(config_file_path, "r") as override_fd:
+            with open(config_file_path) as override_fd:
                 override_content = override_fd.read()
             override_content = yaml.load(override_content, Loader=Loader)
             final_content = merge_contents(final_content, override_content)
-        self.model = Model.parse_obj(final_content)
+        if keyisset("Topics", final_content) and keyisset(
+            "Topics", final_content["Topics"]
+        ):
+            for topic in final_content["Topics"]["Topics"]:
+                if keyisset("Settings", topic):
+                    topic["Settings"] = (
+                        TopicsSettings()
+                        .parse_obj(topic["Settings"])
+                        .dict(
+                            by_alias=True,
+                            exclude_none=True,
+                            exclude_unset=True,
+                            exclude_defaults=True,
+                        )
+                    )
 
+        self.model = Model.parse_obj(final_content)
         if not self.model.Topics and not self.model.ACLs:
             raise KeyError("You must define at least one of ACLs or Topics")
         self.set_globals()
 
     def set_globals(self):
         """
-        Method to set the global settings
+        Method to set the global new_settings
         """
         self.globals_config.update(
             {
@@ -195,28 +217,56 @@ class KafkaStack(object):
             }
         )
 
-    def add_attribute_schema(
-        self,
-        topic_name,
-        registry_url,
-        registry_username,
-        registry_password,
-        registry_userinfo,
-        schema_class,
-        attribute,
-        subject_suffix,
+    def define_schema_definition_path(
+        self, topic_name: str, subject_suffix: str, attribute: TopicSchemaDef
     ):
+        file_name = None
         if isinstance(attribute.Definition, str):
             try:
-                with open(attribute.Definition, "r") as definition_fd:
+                with open(attribute.Definition) as definition_fd:
+                    file_name = attribute.Definition
                     definition = json.dumps(json.loads(definition_fd.read()))
             except FileNotFoundError:
                 print("Failed to load file, using string literal as definition")
                 definition = attribute.Definition
         else:
             definition = attribute.Definition
+            file_name = f"{topic_name}{SerializerDef[attribute.Serializer.name].value}{subject_suffix}Schema.json"
+
+        if self.model.Schemas.S3Store and file_name:
+            s3_file_path = (
+                f"{self.model.Schemas.S3Store.PrefixPath}{FILE_PREFIX}{file_name}"
+            )
+            session = boto3.session.Session()
+            resource = session.resource("s3")
+            bucket = resource.Bucket(self.model.Schemas.S3Store.BucketName)
+            s3_file = bucket.Object(key=s3_file_path)
+            s3_file.put(Body=definition, ContentType="application/json")
+            s3_uri = f"s3://{s3_file.bucket_name}/{s3_file.key}"
+            print(f"Uploaded schema {file_name} to {s3_uri}")
+            return s3_uri
+        else:
+            return definition
+
+    def add_attribute_schema(
+        self,
+        topic_name: str,
+        registry_url,
+        registry_username,
+        registry_password,
+        registry_userinfo,
+        schema_class,
+        attribute: TopicSchemaDef,
+        subject_suffix: str,
+        deletion_policy,
+    ):
+
+        definition = self.define_schema_definition_path(
+            topic_name, subject_suffix, attribute
+        )
         topic_schema_r = schema_class(
             f"{topic_name}{SerializerDef[attribute.Serializer.name].value}{subject_suffix}Schema",
+            DeletionPolicy=deletion_policy,
             SerializeAttribute=subject_suffix,
             Serializer=SerializerDef[attribute.Serializer.name].value,
             Definition=definition,
@@ -262,7 +312,7 @@ class KafkaStack(object):
             registry_url = schema_config["RegistryUrl"]
         else:
             raise KeyError(
-                "RegistryUrl is not defined in Schema settings nor in Schemas"
+                "RegistryUrl is not defined in Schema new_settings nor in Schemas"
             )
 
         if self.model.Schemas and self.model.Schemas.RegistryUsername:
@@ -286,6 +336,25 @@ class KafkaStack(object):
         else:
             registry_userinfo = Ref(AWS_NO_VALUE)
 
+        if keyisset("DeletionPolicy", schema_config):
+            deletion_policy = DeletionPolicy[schema_config["DeletionPolicy"].name].value
+            if (
+                self.model.Schemas
+                and self.model.Schemas.DeletionPolicy
+                and DeletionPolicy[self.model.Schemas.DeletionPolicy.name].value
+                != deletion_policy
+            ):
+                print(
+                    f"WARNING !_! Schema for topic {topic_name} overrides global "
+                    f"{DeletionPolicy[self.model.Schemas.DeletionPolicy.name].value} policy with {deletion_policy}"
+                )
+        elif self.model.Schemas and self.model.Schemas.DeletionPolicy:
+            deletion_policy = DeletionPolicy[
+                self.model.Schemas.DeletionPolicy.name
+            ].value
+        else:
+            deletion_policy = "Retain"
+
         if registry_userinfo and not isinstance(registry_userinfo, Ref):
             registry_username = Ref(AWS_NO_VALUE)
             registry_password = Ref(AWS_NO_VALUE)
@@ -300,6 +369,7 @@ class KafkaStack(object):
                 schema_class,
                 schema_definition.Key,
                 "key",
+                deletion_policy,
             )
         if schema_definition.Value:
             self.add_attribute_schema(
@@ -311,10 +381,11 @@ class KafkaStack(object):
                 schema_class,
                 schema_definition.Value,
                 "value",
+                deletion_policy,
             )
 
     def render_topics(self):
-        if not self.model.Topics or not self.model.Topics.Topics:
+        if not self.model or not self.model.Topics or not self.model.Topics.Topics:
             return
         function_name = None
         if self.model.Topics.FunctionName:
@@ -339,13 +410,23 @@ class KafkaStack(object):
                     else topic.ReplicationFactor,
                 }
             )
-            if not keyisset("Settings", topic_cfg) and keypresent(
+            if keypresent("Settings", topic_cfg) and not keyisset(
                 "Settings", topic_cfg
             ):
                 del topic_cfg["Settings"]
+            else:
+                topic_cfg["Settings"] = json.loads(
+                    topic.Settings.json(
+                        by_alias=True,
+                        exclude_none=True,
+                        exclude_unset=True,
+                        exclude_defaults=True,
+                    )
+                )
             topic_title_raw = topic.Name.__root__
             topic_title = topic_title_raw.replace("-", "").title()
             topic_title = NONALPHANUM.sub("", topic_title)
+
             if topic.Schema and keyisset("Schema", topic_cfg):
                 self.add_topic_schema(topic_title, topic.Schema)
                 del topic_cfg["Schema"]
