@@ -4,9 +4,12 @@
 
 """Module to handle the custom resource for Schemas"""
 
+import json
 import logging
+import re
 
 from aws_cfn_custom_resource_resolve_parser import handle
+from boto3.session import Session
 from cfn_resource_provider import ResourceProvider
 from kafka_schema_registry_admin.kafka_schema_registry_admin import SchemaRegistry
 
@@ -16,11 +19,51 @@ LOG = logging.getLogger(__name__)
 LOG.setLevel(logging.INFO)
 
 
+def pull_from_s3(s3_uri: str) -> str:
+    """
+    Function to pull the definition from S3 and return the JSON String
+    :param s3_uri:
+    :return: The definition
+    """
+    bucket_re = re.compile(r"s3://(?P<bucket>[a-z0-9-.]+)/(?P<key>[\S]+$)")
+    parts = bucket_re.match(s3_uri)
+    if not parts:
+        raise ValueError(
+            "The S3 URI is not valid", s3_uri, "Must match", bucket_re.pattern
+        )
+    bucket_name = parts.group("bucket")
+    key = parts.group("key")
+    session = Session()
+    s3_resource = session.resource("s3")
+    schema_s3_obj = s3_resource.Object(bucket_name, key)
+    body = schema_s3_obj.get()["Body"].read().decode("utf-8")
+    try:
+        body_obj = json.loads(body)
+        LOG.info(f"Successfully retrieved schema from {s3_uri}{schema_s3_obj.e_tag}")
+        return json.dumps(body_obj)
+    except Exception as error:
+        LOG.exception(error)
+        raise
+
+
+def import_definition(schema_definition: str) -> str:
+    """
+    Checks whether the definition is a S3 URI or not. If yes, tries to pull from S3.
+    """
+    if schema_definition.startswith(r"s3://"):
+        LOG.info(f"Attempting to retrieve file from S3 {schema_definition}")
+        return pull_from_s3(schema_definition)
+    else:
+        LOG.info("Using definition as-is.")
+        return schema_definition
+
+
 class KafkaSchema(ResourceProvider):
+    """
+    Class for custom resource to handle a Kafka Schema
+    """
+
     def __init__(self):
-        """
-        Init method
-        """
         self.cluster_info = {}
         super(KafkaSchema, self).__init__()
         self.request_schema = EwsKafkaSchema.schema()
@@ -66,7 +109,7 @@ class KafkaSchema(ResourceProvider):
             subject = self.get("Subject")
             serializer = self.get("Serializer")
             compatibility = self.get("CompatibilityMode")
-            schema_def = self.get("Definition")
+            schema_def = import_definition(self.get("Definition"))
             schema = registry.post_subject_version(
                 subject, schema_def, schema_type=serializer
             )
@@ -77,6 +120,7 @@ class KafkaSchema(ResourceProvider):
             self.physical_resource_id = subject
             self.success("Schema version created")
         except Exception as error:
+            LOG.exception(error)
             self.physical_resource_id = "could-not-create"
             self.fail(str(error))
 
@@ -88,7 +132,7 @@ class KafkaSchema(ResourceProvider):
             registry = self.set_registry()
             subject = self.get("Subject")
             serializer = self.get("Serializer")
-            schema_def = self.get("Definition")
+            schema_def = import_definition(self.get("Definition"))
             compatible = registry.post_compatibility_subjects_versions(
                 subject_name=subject,
                 version_id="latest",
@@ -105,6 +149,7 @@ class KafkaSchema(ResourceProvider):
             self.set_attribute("Id", schema["id"])
             self.success("New schema version created")
         except Exception as error:
+            LOG.exception(error)
             self.fail(str(error))
 
     def delete(self):
