@@ -90,6 +90,7 @@ class KafkaTopic(ResourceProvider):
         if not self.get("PartitionsCount") >= 1:
             self.fail("The number of partitions must be a strictly positive value >= 1")
         try:
+            LOG.info(f'{self.get("Name")} - {self.get("Settings")}')
             topic_name = create_new_kafka_topic(
                 self.get("Name"),
                 self.get("PartitionsCount"),
@@ -129,6 +130,7 @@ class KafkaTopic(ResourceProvider):
                 self.get("Name"),
                 self.get("PartitionsCount"),
                 self.cluster_info,
+                settings=self.get("Settings"),
             )
         except Exception as error:
             self.fail(str(error))
@@ -149,6 +151,10 @@ class KafkaTopic(ResourceProvider):
         try:
             self.define_cluster_info()
             delete_topic(self.get("Name"), self.cluster_info)
+        except errors.UnknownTopicOrPartitionError:
+            self.success(
+                f"Topic {self.get_attribute('Name')} does not exist. Nothing to delete."
+            )
         except Exception as error:
             self.fail(
                 f"Failed to delete topic {self.get_attribute('Name')}. {str(error)}"
@@ -156,6 +162,12 @@ class KafkaTopic(ResourceProvider):
 
 
 def lambda_handler(event, context):
+    """
+    AWS Lambda Function handler for topics management
+
+    :param dict event:
+    :param dict context:
+    """
     provider = KafkaTopic()
     provider.handle(event, context)
 
